'use strict';

var Task = require('data.task');
var Either = require('data.either');

var _require = require('control.monads'),
    liftM2 = _require.liftM2;

var _require2 = require('retry-task'),
    delayedRetry = _require2.delayedRetry;

var eitherToTask = require('../utils/either-to-task');

var _require3 = require('./url-builder'),
    buildVideoPageUrl = _require3.buildVideoPageUrl;

var request = require('../utils/request');

var _require4 = require('../error-handler'),
    videoPageError = _require4.videoPageError;

var extractToken = function extractToken(html, regex) {
  return Either.fromNullable(regex.exec(html)).chain(function (m) {
    return Either.fromNullable(m[1]);
  }).map(function (token) {
    return decodeURIComponent(token);
  });
};

var extractSessionToken = function extractSessionToken(html) {
  return extractToken(html, /'XSRF_TOKEN'\s*\n*:\s*\n*"(.*)"/i).leftMap(function (_) {
    return videoPageError({
      component: 'session-store',
      operation: 'extractSessionToken',
      html: html });
  });
};

var extractCommentsToken = function extractCommentsToken(html) {
  return extractToken(html, /'COMMENTS_TOKEN'\s*\n*:\s*\n*"([^"]+)"/i).leftMap(function (_) {
    return videoPageError({
      component: 'session-store',
      operation: 'extractCommentsToken',
      html: html });
  });
};

// TODO: make # retries configurable
var withRetries = delayedRetry(3, function (n) {
  return n * n * 500;
});

var fetchVideoPage = function fetchVideoPage(videoId) {
  return request(buildVideoPageUrl(videoId));
};

var fetchVideoPageWithRetries = function fetchVideoPageWithRetries(videoId) {
  return withRetries(function () {
    return fetchVideoPage(videoId);
  });
};

var buildSession = function buildSession(sessionToken, commentsToken) {
  return { sessionToken: sessionToken, commentsToken: commentsToken };
};

var getSession = function getSession(videoId) {
  return fetchVideoPageWithRetries(videoId).chain(function (html) {
    return liftM2(buildSession, eitherToTask(extractSessionToken(html)), eitherToTask(extractCommentsToken(html)));
  });
};

// TODO: make cacheTtl configurable
var cacheTtl = 1000 * 60 * 5; // 5 minutes
var cache = {};

/*
 * NOTE: this function is impure (shame on me), but I don't know how else to
 *       memoize lazy async operations.
 */
var memoizedGetSession = function memoizedGetSession(videoId) {
  var cached = cache[videoId];
  if (cached && cached.maxAge > Date.now()) {
    return Task.of(cached.data);
  }

  return getSession(videoId).map(function (res) {
    return (cache[videoId] = {
      data: res,
      maxAge: Date.now() + cacheTtl
    }).data;
  });
};

module.exports = memoizedGetSession;